@@ -14,4 +14,7 @@
 #endif
 
 BOOL WINAPI ErrorCheckMME(MMRESULT ret);
-BOOL WINAPI RaiseExceptionCheck(BOOL bSuccess);
\ No newline at end of file
+BOOL WINAPI RaiseExceptionCheck(BOOL bSuccess);
+BOOL WINAPI ExceptionCheckHRESULT(HRESULT res);
+
+typedef PyObject* PyOb;
\ No newline at end of file
