@@ -3,12 +3,10 @@
     Copyright (c) 2022 Itzsten
 */
 
-// Mainly developed for my YouTube channel, however functions asked to be added probably will.
-// Constant definitions? Meh.
-
 #include "utils.h"
 
 #pragma comment(lib, "gdi32.lib")
+#pragma comment(lib, "msimg32.lib")
 
 static PyObject* PyPatBlt(PyObject* self, PyObject* args) {
     //@description@ Paints a rectangle on the specified device context by mixing the current brush with existing colors according to the rop argument.@@bool
@@ -927,7 +925,7 @@ static PyObject* PyCreatePenIndirect(PyObject* self, PyObject* args) {
 static PyObject* PyExtCreatePen(PyObject* self, PyObject* args) {
     //@description@ Creates a logical cosmetic or geometric pen that has the specified style, width, and brush attributes.@@HPEN
     //@args@ style|int|The type can be ``PS_GEOMETRIC or PS_COSMETIC``, combine it with the OR operator with the style that can be any of the following values; ``PS_ALTERNATE, PS_SOLID, PS_DASH, PS_DOT, PS_DASHDOT, PS_DASHDOTDOT, PS_NULL, PS_USERSTYLE, PS_INSIDEFRAME``, combine it with the OR operator if the pen type is geometric with any of the following values (end cap); ``PS_ENDCAP_ROUND, PS_ENDCAP_SQUARE, PS_ENDCAP_FLAT``, if it is geometrical you should also combine it with the OR operator with any of the following values (join); ``PS_JOIN_BEVEL, PS_JOIN_MITER, PS_JOIN_ROUND``.@@cWidth|int|The width of the pen. If the pen type is ``PS_COSMETIC`` it should be set to 1.@@logBrush|tuple|Tuple containing 3 elements; style, color and a hatch brush.@@pstyle|tuple|Tuple of maximum 16 ints that specifies an user-defined style, or None.
-    DWORD style, width;
+    DWORD style; DWORD width;
     PyObject* pyLb;
     PyObject* pstyle;
     DWORD pstyleR[16];
@@ -976,7 +974,7 @@ static PyObject* PyExtCreatePen(PyObject* self, PyObject* args) {
 static PyObject* PyFillRgn(PyObject* self, PyObject* args) {
     //@description@ Fills the specified region using the given brush.@@bool
     //@args@ hdc|HDC|Handle to a device context@@rgn|HRGN|Region to be filled@@brush|HBRUSH|Brush to paint the region with
-    LONG hdc, hrgn, brush;
+    LONG hdc; LONG hrgn; LONG brush;
     if (!PyArg_ParseTuple(args, "lll", &hdc, &hrgn, &brush)) {
         return NULL;
     }
@@ -1002,7 +1000,8 @@ static PyObject* PyFlattenPath(PyObject* self, PyObject* args) {
 static PyObject* PyFrameRgn(PyObject* self, PyObject* args) {
     //@description@ Draws a border around the specified region by using the specified brush.@@bool
     //@args@ hdc|HDC|Handle to a device context@@rgn|HRGN|Handle to a region to be drawn a border around it@@hbr|HBRUSH|Brush to paint the border with@@w|int|Width of vertical brush strokes.@@h|int|Height of horizontal brush strokes.
-    LONG hdc, hrgn, brush;
+    LONG hdc;
+    LONG hrgn; LONG brush;
     INT w, h;
     if (!PyArg_ParseTuple(args, "lllii", &hdc, &hrgn, &brush, &w, &h)) {
         return NULL;
@@ -1014,7 +1013,8 @@ static PyObject* PyFrameRgn(PyObject* self, PyObject* args) {
     return Py_BuildValue("O", PyBool_FromLong(TRUE));
 }
 static PyObject* PyInvertRgn(PyObject* self, PyObject* args) {
-    LONG hdc, hrgn;
+    LONG hdc;
+    LONG hrgn;
     if (!PyArg_ParseTuple(args, "ll", &hdc, &hrgn)) {
         return NULL;
     }
@@ -1233,10 +1233,8 @@ static PyObject* PyDib_GetSectionData(PY_DIBDATA* self, PyObject* args) {
 
     CHAR* result = HeapAlloc(hHeap, 0, sizeof(CHAR) * len * 3);
 
-    printf("%ld x %ld (%ld)\n", rgb->nWidth, rgb->nHeight, rgb->nSize);
 
     for (INT i = 0; i < len; i+= 3) {
-        printf("%d", i);
 
         ra = i != 0 ? i / 3 : 0;
 
@@ -1317,7 +1315,783 @@ static PyTypeObject PyDib_Types = {
     PyDib_new,                 /* tp_new */
 };
 
+static PyObject* PyPtInRegion(PyObject* _, PyObject* args) {
+    LONG hrgn;
+    INT x;
+    INT y;
+    if (!PyArg_ParseTuple(args, "lii", &hrgn, &x, &y)) return NULL;
+    return Py_BuildValue("O", PyBool_FromLong(PtInRegion(hrgn, x, y)));
+}
+static PyObject* PyCreateFontA(PyObject* _, PyObject* args) {
+    int height;
+    int width;
+    int escapement;
+    int orientation;
+    int weight;
+    PyObject* obItalic;
+    PyObject* obUnderline;
+    PyObject* obStrikeOut;
+    DWORD charSet;
+    DWORD outPrecision;
+    DWORD clipPrecision;
+    DWORD quality;
+    DWORD pitchAndFamily;
+    LPSTR fontName;
+    if (!PyArg_ParseTuple(args, "iiiiiOOOkkkkks", &height, &width, &escapement, &orientation, &weight, &obItalic, &obUnderline, &obStrikeOut,
+        &charSet, &outPrecision, &clipPrecision, &quality, &pitchAndFamily, &fontName
+    )) return NULL;
+    BOOL italic = PyObject_IsTrue(obItalic);
+    BOOL underline = PyObject_IsTrue(obUnderline);
+    BOOL strikeOut = PyObject_IsTrue(obStrikeOut);
+
+    HFONT res = CreateFontA(height, width, escapement, orientation, weight, italic, underline, strikeOut, charSet, outPrecision,
+        clipPrecision, quality, pitchAndFamily, fontName
+    );
+    if (res == NULL) {
+        PyErr_SetFromWindowsErr(87);
+        return NULL;
+    }
+    return Py_BuildValue("L", res);
+}
+static PyObject* PyCreateFontW(PyObject* _, PyObject* args) {
+    int height;
+    int width;
+    int escapement;
+    int orientation;
+    int weight;
+    PyObject* obItalic;
+    PyObject* obUnderline;
+    PyObject* obStrikeOut;
+    DWORD charSet;
+    DWORD outPrecision;
+    DWORD clipPrecision;
+    DWORD quality;
+    DWORD pitchAndFamily;
+    LPWSTR fontName;
+    PyObject* fontOb;
+    if (!PyArg_ParseTuple(args, "iiiiiOOOkkkkkO", &height, &width, &escapement, &orientation, &weight, &obItalic, &obUnderline, &obStrikeOut,
+        &charSet, &outPrecision, &clipPrecision, &quality, &pitchAndFamily, &fontOb
+    )) return NULL;
+    BOOL italic = PyObject_IsTrue(obItalic);
+    BOOL underline = PyObject_IsTrue(obUnderline);
+    BOOL strikeOut = PyObject_IsTrue(obStrikeOut);
+    fontName = PyUnicode_AsWideCharString(fontOb, NULL);
+
+    HFONT res = CreateFontW(height, width, escapement, orientation, weight, italic, underline, strikeOut, charSet, outPrecision,
+        clipPrecision, quality, pitchAndFamily, fontName
+    );
+    if (res == NULL) {
+        PyErr_SetFromWindowsErr(87);
+        return NULL;
+    }
+    return Py_BuildValue("L", res);
+}
+static PyObject* PyGetTextExtentPoint32A(PyObject* _, PyObject* args) {
+    HDC hdc;
+    LPSTR str;
+    if (!PyArg_ParseTuple(args, "ls", &hdc, &str)) return NULL;
+    INT c = strlen(str);
+    SIZE sz;
+    if (!GetTextExtentPoint32A(hdc, str, c, &sz)) {
+        PyErr_SetFromWindowsErr(87);
+        return NULL;
+    }
+    return Py_BuildValue("(ll)", sz.cx, sz.cy);
+}
+static PyObject* PyGetTextExtentPoint32W(PyObject* _, PyObject* args) {
+    HDC hdc;
+    LPWSTR str;
+    PyObject* obStr;
+    if (!PyArg_ParseTuple(args, "lO", &hdc, &obStr)) return NULL;
+    str = PyUnicode_AsWideCharString(obStr, NULL);
+    INT c = wcslen(str);
+    SIZE sz;
+    if (!GetTextExtentPoint32W(hdc, str, c, &sz)) {
+        PyErr_SetFromWindowsErr(87);
+        return NULL;
+    }
+    return Py_BuildValue("(ll)", sz.cx, sz.cy);
+}
+static PyObject* PyRectInRegion(PyObject* _, PyObject* args) {
+    HRGN rgn;
+    RECT rc;
+    if (!PyArg_ParseTuple(args, "l(iiii)", &rgn, &rc.left, &rc.top, &rc.right, &rc.bottom)) return NULL;
+    return Py_BuildValue("O", PyBool_FromLong(RectInRegion(rgn, &rc)));
+}
+static PyObject* PyRectVisible(PyObject* _, PyObject* args) {
+    HDC hdc;
+    RECT rc;
+    if (!PyArg_ParseTuple(args, "l(iiii)", &hdc, &rc.left, &rc.top, &rc.right, &rc.bottom)) return NULL;
+    return Py_BuildValue("O", PyBool_FromLong(RectVisible(hdc, &rc)));
+}
+static PyObject* PyPtVisible(PyObject* _, PyObject* args) {
+    HDC hdc;
+    INT x;
+    INT y;
+    if (!PyArg_ParseTuple(args, "lii", &hdc, &x, &y)) return NULL;
+    return Py_BuildValue("O", PyBool_FromLong(PtVisible(hdc, x, y)));
+}
+static PyObject* PyPolyTextOutA(PyObject* _, PyObject* args) {
+    PyObject* ppt;
+    HDC hdc;
+    if (!PyArg_ParseTuple(args, "lO", &hdc, &ppt)) return NULL;
+    if ((!PyTuple_Check(ppt)) || (PyTuple_GET_SIZE(ppt) == 0)) {
+        PyErr_SetString(PyExc_TypeError, "argument 2: excepted tuple of additional tuples describing how PolyTextOut should draw the string");
+        return NULL;
+    }
+    INT count = PyTuple_GET_SIZE(ppt);
+    POLYTEXTA* arr = malloc(count * sizeof(POLYTEXTA));
+
+    for (INT i = 0; i < count; i++) {
+        PyObject* st = PyTuple_GetItem(ppt, i);
+        if (!PyTuple_Check(st)) { free(arr); PyErr_SetString(PyExc_ValueError, "argument 2: excepted tuple of additional tuples describing how PolyTextOut should draw the string");  return NULL; }
+        if (PyTuple_GET_SIZE(st) != 6) { free(arr); PyErr_SetString(PyExc_ValueError, "argument 2: tuples must be of length 6"); return NULL; }
+
+        arr[i].x = PyLong_AsLong(PyTuple_GetItem(st, 0));
+        arr[i].y = PyLong_AsLong(PyTuple_GetItem(st, 1));
+        arr[i].lpstr = PyString_ToCharArr(PyTuple_GetItem(st, 2));
+        arr[i].uiFlags = PyLong_AsUnsignedLong(PyTuple_GetItem(st, 3));
+        arr[i].n = strlen(arr[i].lpstr);
+        if (PyTuple_GetItem(st, 4) != Py_None) {
+            PyObject* rect = PyTuple_GetItem(st, 4);
+            if (!PyTuple_Check(rect) || (PyTuple_GET_SIZE(rect) != 4)) {
+                PyErr_SetString(PyExc_ValueError, "argument 2: index 4: excepted a tuple containing four integers, or None");
+                free(arr);
+                return NULL;
+            }
+            arr[i].rcl.left   = PyLong_AsLong(PyTuple_GetItem(rect, 0));
+            arr[i].rcl.top    = PyLong_AsLong(PyTuple_GetItem(rect, 1));
+            arr[i].rcl.right  = PyLong_AsLong(PyTuple_GetItem(rect, 2));
+            arr[i].rcl.bottom = PyLong_AsLong(PyTuple_GetItem(rect, 3));
+        }
+        PyObject* pdx = PyTuple_GetItem(st, 5);
+        if ((!PyList_Check(pdx)) || (PyList_GET_SIZE(pdx) == 0)) {
+            PyErr_SetString(PyExc_ValueError, "argument 2: index 5: excepted a list object describing the characters width");
+            free(arr);
+            return NULL;
+        }
+        SSIZE_T sz = PyList_GET_SIZE(pdx);
+        INT* pwArr = malloc(sizeof(INT) * sz);
+        for (SSIZE_T i2 = 0; i2 < sz; i2++) {
+            pwArr[i2] = PyLong_AsLong( PyList_GetItem(pdx, i2) );
+        }
+        arr[i].pdx = pwArr;
+    }
+
+    BOOL result = PolyTextOutA(hdc, arr, count);
+
+    for (INT i = 0; i < count; i++) {
+        free(arr[i].pdx);
+    }
+    free(arr);
+
+    if (!result) {
+        PyErr_SetFromWindowsErr(87);
+        return NULL;
+    }
+    return Py_BuildValue("i", 0);
+}
+static PyObject* PyPolyTextOutW(PyObject* _, PyObject* args) {
+    PyObject* ppt;
+    HDC hdc;
+    if (!PyArg_ParseTuple(args, "lO", &hdc, &ppt)) return NULL;
+    if ((!PyTuple_Check(ppt)) || (PyTuple_GET_SIZE(ppt) == 0)) {
+        PyErr_SetString(PyExc_TypeError, "argument 2: excepted tuple of additional tuples describing how PolyTextOut should draw the string");
+        return NULL;
+    }
+    INT count = PyTuple_GET_SIZE(ppt);
+    POLYTEXTW* arr = malloc(count * sizeof(POLYTEXTW));
+
+    for (INT i = 0; i < count; i++) {
+        PyObject* st = PyTuple_GetItem(ppt, i);
+        if (!PyTuple_Check(st)) { free(arr); PyErr_SetString(PyExc_ValueError, "argument 2: excepted tuple of additional tuples describing how PolyTextOut should draw the string");  return NULL; }
+        if (PyTuple_GET_SIZE(st) != 6) { free(arr); PyErr_SetString(PyExc_ValueError, "argument 2: tuples must be of length 6"); return NULL; }
+
+        arr[i].x = PyLong_AsLong(PyTuple_GetItem(st, 0));
+        arr[i].y = PyLong_AsLong(PyTuple_GetItem(st, 1));
+        arr[i].lpstr = PyUnicode_AsWideCharString(PyTuple_GetItem(st, 2), NULL);
+        arr[i].uiFlags = PyLong_AsUnsignedLong(PyTuple_GetItem(st, 3));
+        arr[i].n = wcslen(arr[i].lpstr);
+        if (PyTuple_GetItem(st, 4) != Py_None) {
+            PyObject* rect = PyTuple_GetItem(st, 4);
+            if (!PyTuple_Check(rect) || (PyTuple_GET_SIZE(rect) != 4)) {
+                PyErr_SetString(PyExc_ValueError, "argument 2: index 4: excepted a tuple containing four integers, or None");
+                free(arr);
+                return NULL;
+            }
+            arr[i].rcl.left = PyLong_AsLong(PyTuple_GetItem(rect, 0));
+            arr[i].rcl.top = PyLong_AsLong(PyTuple_GetItem(rect, 1));
+            arr[i].rcl.right = PyLong_AsLong(PyTuple_GetItem(rect, 2));
+            arr[i].rcl.bottom = PyLong_AsLong(PyTuple_GetItem(rect, 3));
+        }
+        PyObject* pdx = PyTuple_GetItem(st, 5);
+        if ((!PyList_Check(pdx)) || (PyList_GET_SIZE(pdx) == 0)) {
+            PyErr_SetString(PyExc_ValueError, "argument 2: index 5: excepted a list object describing the characters width");
+            free(arr);
+            return NULL;
+        }
+        SSIZE_T sz = PyList_GET_SIZE(pdx);
+        INT* pwArr = malloc(sizeof(INT) * sz);
+        for (SSIZE_T i2 = 0; i2 < sz; i2++) {
+            pwArr[i2] = PyLong_AsLong(PyList_GetItem(pdx, i2));
+        }
+        arr[i].pdx = pwArr;
+    }
+
+    BOOL result = PolyTextOutW(hdc, arr, count);
+
+    for (INT i = 0; i < count; i++) {
+        free(arr[i].pdx);
+        PyMem_Free(arr[i].lpstr);
+    }
+    free(arr);
+
+    if (!result) {
+        PyErr_SetFromWindowsErr(87);
+        return NULL;
+    }
+    return Py_BuildValue("i", 0);
+}
+static PyObject* PyCreatePolyPolygonRgn(PyObject* _, PyObject* args) {
+    PyObject* pts;
+    INT mode;
+    if (!PyArg_ParseTuple(args, "Oi", &pts, &mode)) return NULL;
+    if ((!PyTuple_Check(pts)) || (PyTuple_GET_SIZE(pts) == 0)) {
+        PyErr_SetString(PyExc_TypeError, "argument 1: excepted tuple containing tuples containing a series of points (x, y) describing the polygons");
+        return NULL;
+    }
+    INT sz = PyTuple_GET_SIZE(pts);
+    INT totalPoints = 0;
+    INT totalPolygons = sz;
+    for (INT i = 0; i < sz; i++) {
+        PyObject* item = PyTuple_GetItem(pts, i);
+        if ((!PyTuple_Check(item)) || (PyTuple_GET_SIZE(item) == 0)) {
+            PyErr_SetString(PyExc_TypeError, "argument 1: excepted tuple containing tuples containing a series of points (x, y) describing the polygons");
+            return NULL;
+        }
+        for (INT i2 = 0; i2 < PyTuple_GET_SIZE(item); i2++) {
+            PyObject* item2 = PyTuple_GetItem(item, i2);
+            if ((!PyTuple_Check(item2)) || (PyTuple_GET_SIZE(item2) != 2)) {
+                PyErr_SetString(PyExc_TypeError, "argument 1: excepted tuple containing tuples containing a series of points (x, y) describing the polygons");
+                return NULL;
+            }
+            PyLong_AsLong(PyTuple_GetItem(item2, 0));
+            if (PyErr_Occurred()) return NULL;
+            PyLong_AsLong(PyTuple_GetItem(item2, 1));
+            if (PyErr_Occurred()) return NULL;
+        }
+        totalPoints += PyTuple_GET_SIZE(item);
+    }
+
+    POINT* ppm = malloc(sizeof(POINT) * totalPoints);
+    INT* polyCounts = malloc(sizeof(INT) * sz);
+    
+    INT verticeCount = 0;
+    for (INT i = 0; i < sz; i++) {
+        PyObject* polygon = PyTuple_GetItem(pts, i);
+        INT vertices = PyTuple_GET_SIZE(polygon);
+        polyCounts[i] = vertices;
+
+        for (INT j = 0; j < vertices; j++) {
+            PyObject* point = PyTuple_GetItem(polygon, j);
+            ppm[verticeCount].x = PyLong_AsLong(PyTuple_GET_ITEM(point, 0));
+            ppm[verticeCount].y = PyLong_AsLong(PyTuple_GET_ITEM(point, 1));
+            verticeCount += 1;
+        }
+    }
+
+    HRGN result = CreatePolyPolygonRgn(ppm, polyCounts, sz, mode);
+    free(ppm);
+    free(polyCounts);
+    if (!result) {
+        PyErr_SetFromWindowsErr(87);
+        return NULL;
+    }
+    return Py_BuildValue("L", result);
+}
+static PyObject* PyPolyBezier(PyObject* _, PyObject* args) {
+    HDC hdc;
+    PyObject* beizerArr;
+    if (!PyArg_ParseTuple(args, "lO", &hdc, &beizerArr)) return NULL;
+    if ((!PyTuple_Check(beizerArr)) || (PyTuple_GET_SIZE(beizerArr) < 4)) {
+        PyErr_SetString(PyExc_TypeError, "argument 2: must be a sequence of points (x, y) with minimun length as 4");
+        return NULL;
+    }
+    DWORD sz = PyTuple_GET_SIZE(beizerArr);
+    for (INT i = 0; i < sz; i++) {
+        PyObject* item = PyTuple_GetItem(beizerArr, i);
+        if ((!PyTuple_Check(item)) || (PyTuple_GET_SIZE(item) != 2)) {
+            PyErr_SetString(PyExc_ValueError, "argument 2: must contain at least 4 tuple objects with 2 integers (x, y)");
+            return NULL;
+        }
+        PyLong_AsLong(PyTuple_GetItem(item, 0));
+        if (PyErr_Occurred()) return NULL;
+        PyLong_AsLong(PyTuple_GetItem(item, 1));
+        if (PyErr_Occurred()) return NULL;
+    }
+    POINT* ppm = malloc(sizeof(POINT) * sz);
+    for (INT i = 0; i < sz; i++) {
+        PyObject* bPoint = PyTuple_GetItem(beizerArr, i);
+        ppm[i].x = PyLong_AsLong(PyTuple_GetItem(bPoint, 0));
+        ppm[i].y = PyLong_AsLong(PyTuple_GetItem(bPoint, 1));
+    }
+    BOOL res = PolyBezier(hdc, ppm, sz);
+    free(ppm);
+    if (!res) {
+        PyErr_SetFromWindowsErr(87);
+        return NULL;
+    }
+    return Py_BuildValue("O", Py_None);
+}
+static PyObject* PyPolyBezierTo(PyObject* _, PyObject* args) {
+    HDC hdc;
+    PyObject* beizerArr;
+    if (!PyArg_ParseTuple(args, "lO", &hdc, &beizerArr)) return NULL;
+    if ((!PyTuple_Check(beizerArr)) || (PyTuple_GET_SIZE(beizerArr) < 3)) {
+        PyErr_SetString(PyExc_TypeError, "argument 2: must be a sequence of points (x, y) with minimun length as 3");
+        return NULL;
+    }
+    DWORD sz = PyTuple_GET_SIZE(beizerArr);
+    for (INT i = 0; i < sz; i++) {
+        PyObject* item = PyTuple_GetItem(beizerArr, i);
+        if ((!PyTuple_Check(item)) || (PyTuple_GET_SIZE(item) != 2)) {
+            PyErr_SetString(PyExc_ValueError, "argument 2: must contain at least 3 tuple objects with 2 integers (x, y)");
+            return NULL;
+        }
+        PyLong_AsLong(PyTuple_GetItem(item, 0));
+        if (PyErr_Occurred()) return NULL;
+        PyLong_AsLong(PyTuple_GetItem(item, 1));
+        if (PyErr_Occurred()) return NULL;
+    }
+    POINT* ppm = malloc(sizeof(POINT) * sz);
+    for (INT i = 0; i < sz; i++) {
+        PyObject* bPoint = PyTuple_GetItem(beizerArr, i);
+        ppm[i].x = PyLong_AsLong(PyTuple_GetItem(bPoint, 0));
+        ppm[i].y = PyLong_AsLong(PyTuple_GetItem(bPoint, 1));
+    }
+    BOOL res = PolyBezierTo(hdc, ppm, sz);
+    free(ppm);
+    if (!res) {
+        PyErr_SetFromWindowsErr(87);
+        return NULL;
+    }
+    return Py_BuildValue("O", Py_None);
+}
+static PyObject* PyMoveToEx(PyObject* _, PyObject* args) {
+    HDC hdc;
+    INT x;
+    INT y;
+    if (!PyArg_ParseTuple(args, "lii", &hdc, &x, &y)) return NULL;
+    POINT res;
+    if (RaiseExceptionCheckInvalid(MoveToEx(hdc, x, y, &res))) return NULL;
+    return Py_BuildValue("(ii)", res.x, res.y);
+}
+static PyObject* PyPolygon(PyObject* _, PyObject* args) {
+    HDC hdc;
+    PyObject* polygon;
+    if (!PyArg_ParseTuple(args, "lO", &hdc, &polygon)) return NULL;
+    if ((!PyTuple_Check(polygon)) || (PyTuple_GET_SIZE(polygon) < 2)) {
+        PyErr_SetString(PyExc_TypeError, "argument 2: excepted tuple object containing points (x, y) with minimum length as 2");
+        return NULL;
+    }
+    INT sz = PyTuple_GET_SIZE(polygon);
+    POINT* ppm = malloc(sizeof(POINT) * sz);
+
+    for (INT i = 0; i < sz; i++) {
+        PyObject* vertice = PyTuple_GetItem(polygon, i);
+        if ((!PyTuple_Check(vertice)) || (PyTuple_GET_SIZE(vertice) != 2)) {
+            PyErr_SetString(PyExc_ValueError, "argument 2: excepted tuple object containing points (tuple of two integers x, y)");
+            free(ppm);
+            return NULL;
+        }
+        ppm[i].x = PyLong_AsLong(PyTuple_GET_ITEM(vertice, 0));
+        ppm[i].y = PyLong_AsLong(PyTuple_GET_ITEM(vertice, 1));
+    }
+
+    BOOL result = Polygon(hdc, ppm, sz);
+    free(ppm);
+    if (RaiseExceptionCheckInvalid(result)) return NULL;
+    return Py_BuildValue("O", Py_None);
+}
+static PyObject* PyPolyPolygon(PyObject* _, PyObject* args) {
+    HDC hdc;
+    PyObject* pts;
+    if (!PyArg_ParseTuple(args, "lO", &hdc, &pts)) return NULL;
+    if ((!PyTuple_Check(pts)) || (PyTuple_GET_SIZE(pts) == 0)) {
+        PyErr_SetString(PyExc_TypeError, "argument 1: excepted tuple containing tuples containing a series of points (x, y) describing the polygons");
+        return NULL;
+    }
+    INT sz = PyTuple_GET_SIZE(pts);
+    INT totalPoints = 0;
+    INT totalPolygons = sz;
+    for (INT i = 0; i < sz; i++) {
+        PyObject* item = PyTuple_GetItem(pts, i);
+        if ((!PyTuple_Check(item)) || (PyTuple_GET_SIZE(item) == 0)) {
+            PyErr_SetString(PyExc_TypeError, "argument 1: excepted tuple containing tuples containing a series of points (x, y) describing the polygons");
+            return NULL;
+        }
+        for (INT i2 = 0; i2 < PyTuple_GET_SIZE(item); i2++) {
+            PyObject* item2 = PyTuple_GetItem(item, i2);
+            if ((!PyTuple_Check(item2)) || (PyTuple_GET_SIZE(item2) != 2)) {
+                PyErr_SetString(PyExc_TypeError, "argument 1: excepted tuple containing tuples containing a series of points (x, y) describing the polygons");
+                return NULL;
+            }
+            PyLong_AsLong(PyTuple_GetItem(item2, 0));
+            if (PyErr_Occurred()) return NULL;
+            PyLong_AsLong(PyTuple_GetItem(item2, 1));
+            if (PyErr_Occurred()) return NULL;
+        }
+        totalPoints += PyTuple_GET_SIZE(item);
+    }
+
+    POINT* ppm = malloc(sizeof(POINT) * totalPoints);
+    INT* polyCounts = malloc(sizeof(INT) * sz);
+
+    INT verticeCount = 0;
+    for (INT i = 0; i < sz; i++) {
+        PyObject* polygon = PyTuple_GetItem(pts, i);
+        INT vertices = PyTuple_GET_SIZE(polygon);
+        polyCounts[i] = vertices;
+
+        for (INT j = 0; j < vertices; j++) {
+            PyObject* point = PyTuple_GetItem(polygon, j);
+            ppm[verticeCount].x = PyLong_AsLong(PyTuple_GET_ITEM(point, 0));
+            ppm[verticeCount].y = PyLong_AsLong(PyTuple_GET_ITEM(point, 1));
+            verticeCount += 1;
+        }
+    }
+
+    BOOL result = PolyPolygon(hdc, ppm, polyCounts, sz);
+    free(ppm);
+    free(polyCounts);
+    if (!result) {
+        PyErr_SetFromWindowsErr(87);
+        return NULL;
+    }
+    return Py_BuildValue("O", Py_None);
+}
+static PyObject* PyPolyDraw(PyObject* _, PyObject* args) {
+    HDC hdc;
+    PyObject* beizerArr;
+    PyObject* settingsArr;
+    if (!PyArg_ParseTuple(args, "lOO", &hdc, &beizerArr, &settingsArr)) return NULL;
+    if (!PyTuple_Check(beizerArr)) {
+        PyErr_SetString(PyExc_TypeError, "argument 2: must be a tuple object");
+        return NULL;
+    }
+    if (!PyTuple_Check(settingsArr)) {
+        PyErr_SetString(PyExc_TypeError, "argument 3: must be a tuple object");
+        return NULL;
+    }
+    if (PyTuple_GET_SIZE(beizerArr) != PyTuple_GET_SIZE(settingsArr)) {
+        PyErr_SetString(PyExc_ValueError, "argument 3: must be the same size as argument 2");
+        return NULL;
+    }
+    INT sz = PyTuple_GET_SIZE(beizerArr);
+    for (INT i = 0; i < sz; i++) {
+        PyObject* item = PyTuple_GetItem(beizerArr, i);
+        PyObject* setting = PyTuple_GetItem(settingsArr, i);
+        if ((!PyTuple_Check(item)) || (PyTuple_GET_SIZE(item) != 2)) {
+            PyErr_SetString(PyExc_ValueError, "argument 2: must have sequences of points of length 2 (x, y)");
+            return NULL;
+        }
+        if ((PyLong_AsUnsignedLong(setting)||1) && PyErr_Occurred()) return NULL;
+        if ((PyLong_AsLong(PyTuple_GetItem(item, 0))||1) && PyErr_Occurred()) return NULL;
+        if ((PyLong_AsLong(PyTuple_GetItem(item, 1))||1) && PyErr_Occurred()) return NULL;
+    }
+
+    POINT* ppm = malloc(sizeof(POINT) * sz);
+    BYTE * pj  = malloc(sizeof(BYTE)  * sz);
+
+    for (INT i = 0; i < sz; i++) {
+        PyObject* point = PyTuple_GetItem(beizerArr, i);
+        PyObject* setting = PyTuple_GetItem(settingsArr, i);
+        ppm[i].x = PyLong_AsLong(PyTuple_GetItem(point, 0));
+        ppm[i].y = PyLong_AsLong(PyTuple_GetItem(point, 1));
+        pj[i] = PyLong_AsUnsignedLong(setting);
+    }
+
+    BOOL result = PolyDraw( hdc, ppm, pj, sz );
+
+    free(ppm);
+    free(pj);
+
+    if (RaiseExceptionCheckInvalid(result)) return NULL;
+    return Py_BuildValue("O", Py_None);
+}
+static PyObject* PyPolyline(PyObject* _, PyObject* args) {
+    HDC hdc;
+    PyObject* pArr;
+    if (!PyArg_ParseTuple(args, "lO", &hdc, &pArr)) return NULL;
+    if ((!PyTuple_Check(pArr)) || (PyTuple_GET_SIZE(pArr)==0)) {
+        PyErr_SetString(PyExc_TypeError, "argument 2: excepted tuple object");
+        return NULL;
+    }
+    INT sz = PyTuple_GET_SIZE(pArr);
+    for (INT i = 0; i < sz; i++) {
+        PyObject* item = PyTuple_GetItem(pArr, i);
+        if ((!PyTuple_Check(item)) || (PyTuple_GET_SIZE(item) != 2)) {
+            PyErr_SetString(PyExc_ValueError, "argument 2: must have sequences of points of length 2 (x, y)");
+            return NULL;
+        }
+        if ((PyLong_AsLong(PyTuple_GetItem(item, 0)) || 1) && PyErr_Occurred()) return NULL;
+        if ((PyLong_AsLong(PyTuple_GetItem(item, 1)) || 1) && PyErr_Occurred()) return NULL;
+    }
+    POINT* ppm = malloc(sizeof(POINT) * sz);
+    for (INT i = 0; i < sz; i++) {
+        PyObject* item = PyTuple_GetItem(pArr, i);
+        ppm[i].x = PyLong_AsLong(PyTuple_GetItem(item, 0));
+        ppm[i].y = PyLong_AsLong(PyTuple_GetItem(item, 1));
+    }
+    BOOL result = Polyline(hdc, ppm, sz);
+    free(ppm);
+    if (RaiseExceptionCheckInvalid(result)) return NULL;
+    return Py_BuildValue("O", Py_None);
+}
+static PyObject* PyPolylineTo(PyObject* _, PyObject* args) {
+    HDC hdc;
+    PyObject* pArr;
+    if (!PyArg_ParseTuple(args, "lO", &hdc, &pArr)) return NULL;
+    if ((!PyTuple_Check(pArr)) || (PyTuple_GET_SIZE(pArr) == 0)) {
+        PyErr_SetString(PyExc_TypeError, "argument 2: excepted tuple object");
+        return NULL;
+    }
+    INT sz = PyTuple_GET_SIZE(pArr);
+    for (INT i = 0; i < sz; i++) {
+        PyObject* item = PyTuple_GetItem(pArr, i);
+        if ((!PyTuple_Check(item)) || (PyTuple_GET_SIZE(item) != 2)) {
+            PyErr_SetString(PyExc_ValueError, "argument 2: must have sequences of points of length 2 (x, y)");
+            return NULL;
+        }
+        if ((PyLong_AsLong(PyTuple_GetItem(item, 0)) || 1) && PyErr_Occurred()) return NULL;
+        if ((PyLong_AsLong(PyTuple_GetItem(item, 1)) || 1) && PyErr_Occurred()) return NULL;
+    }
+    POINT* ppm = malloc(sizeof(POINT) * sz);
+    for (INT i = 0; i < sz; i++) {
+        PyObject* item = PyTuple_GetItem(pArr, i);
+        ppm[i].x = PyLong_AsLong(PyTuple_GetItem(item, 0));
+        ppm[i].y = PyLong_AsLong(PyTuple_GetItem(item, 1));
+    }
+    BOOL result = PolylineTo(hdc, ppm, sz);
+    free(ppm);
+    if (RaiseExceptionCheckInvalid(result)) return NULL;
+    return Py_BuildValue("O", Py_None);
+}
+static PyObject* PyPolyPolyline(PyObject* _, PyObject* args) {
+    HDC hdc;
+    PyObject* pts;
+    if (!PyArg_ParseTuple(args, "lO", &hdc, &pts)) return NULL;
+    if ((!PyTuple_Check(pts)) || (PyTuple_GET_SIZE(pts) == 0)) {
+        PyErr_SetString(PyExc_TypeError, "argument 1: excepted tuple containing tuples containing a series of points (x, y) describing the polylines");
+        return NULL;
+    }
+    DWORD sz = PyTuple_GET_SIZE(pts);
+    INT totalPoints = 0;
+    INT totalPolygons = sz;
+    for (INT i = 0; i < sz; i++) {
+        PyObject* item = PyTuple_GetItem(pts, i);
+        if ((!PyTuple_Check(item)) || (PyTuple_GET_SIZE(item) == 0)) {
+            PyErr_SetString(PyExc_TypeError, "argument 1: excepted tuple containing tuples containing a series of points (x, y) describing the polylines");
+            return NULL;
+        }
+        for (INT i2 = 0; i2 < PyTuple_GET_SIZE(item); i2++) {
+            PyObject* item2 = PyTuple_GetItem(item, i2);
+            if ((!PyTuple_Check(item2)) || (PyTuple_GET_SIZE(item2) != 2)) {
+                PyErr_SetString(PyExc_TypeError, "argument 1: excepted tuple containing tuples containing a series of points (x, y) describing the polylines");
+                return NULL;
+            }
+            PyLong_AsLong(PyTuple_GetItem(item2, 0));
+            if (PyErr_Occurred()) return NULL;
+            PyLong_AsLong(PyTuple_GetItem(item2, 1));
+            if (PyErr_Occurred()) return NULL;
+        }
+        totalPoints += PyTuple_GET_SIZE(item);
+    }
+
+    POINT* ppm = malloc(sizeof(POINT) * totalPoints);
+    DWORD* polyCounts = malloc(sizeof(DWORD) * sz);
+
+    INT verticeCount = 0;
+    for (INT i = 0; i < sz; i++) {
+        PyObject* polygon = PyTuple_GetItem(pts, i);
+        DWORD vertices = PyTuple_GET_SIZE(polygon);
+        polyCounts[i] = vertices;
+
+        for (INT j = 0; j < vertices; j++) {
+            PyObject* point = PyTuple_GetItem(polygon, j);
+            ppm[verticeCount].x = PyLong_AsLong(PyTuple_GET_ITEM(point, 0));
+            ppm[verticeCount].y = PyLong_AsLong(PyTuple_GET_ITEM(point, 1));
+            verticeCount += 1;
+        }
+    }
+
+    BOOL result = PolyPolyline(hdc, ppm, polyCounts, sz);
+    free(ppm);
+    free(polyCounts);
+    if (!result) {
+        PyErr_SetFromWindowsErr(87);
+        return NULL;
+    }
+    return Py_BuildValue("O", Py_None);
+}
+static PyObject* PyOffsetRgn(PyObject* _, PyObject* args) {
+    HRGN rgn;
+    INT x;
+    INT y;
+    if (!PyArg_ParseTuple(args, "lii", &rgn, &x, &y)) return NULL;
+    return Py_BuildValue("i", OffsetRgn(rgn, x, y));
+}
+static PyObject* PyOffsetClipRgn(PyObject* _, PyObject* args) {
+    HDC hdc;
+    INT x;
+    INT y;
+    if (!PyArg_ParseTuple(args, "lii", &hdc, &x, &y)) return NULL;
+    return Py_BuildValue("i", OffsetClipRgn(hdc, x, y));
+}
+static PyObject* PyOffsetViewportOrgEx(PyObject* _, PyObject* args) {
+    HDC hdc;
+    INT x;
+    INT y;
+    if (!PyArg_ParseTuple(args, "lii", &hdc, &x, &y)) return NULL;
+    POINT res;
+    if (RaiseExceptionCheckInvalid(OffsetViewportOrgEx(hdc, x, y, &res))) return NULL;
+    return Py_BuildValue("(ii)", res.x, res.y);
+}
+static PyObject* PyOffsetWindowOrgEx(PyObject* _, PyObject* args) {
+    HDC hdc;
+    INT x;
+    INT y;
+    if (!PyArg_ParseTuple(args, "lii", &hdc, &x, &y)) return NULL;
+    POINT res;
+    if (RaiseExceptionCheckInvalid(OffsetWindowOrgEx(hdc, x, y, &res))) return NULL;
+    return Py_BuildValue("(ii)", res.x, res.y);
+}
+static PyObject* PyModifyWorldTransform(PyObject* _, PyObject* args) {
+    HDC hdc;
+    XFORM xf;
+    DWORD dwMode;
+    if (!PyArg_ParseTuple(args, "l(ffffff)k", &hdc, &xf.eM11, &xf.eM12, &xf.eM21, &xf.eM22, &xf.eDx, &xf.eDy, &dwMode)) return NULL;
+    if (RaiseExceptionCheckInvalid(ModifyWorldTransform(hdc, &xf, dwMode))) return NULL;
+    return Py_BuildValue("O", Py_None);
+}
+static PyObject* PySetWorldTransform(PyObject* _, PyObject* args) {
+    HDC hdc;
+    XFORM xf;
+    if (!PyArg_ParseTuple(args, "l(ffffff)", &hdc, &xf.eM11, &xf.eM12, &xf.eM21, &xf.eM22, &xf.eDx, &xf.eDy)) return NULL;
+    if (RaiseExceptionCheckInvalid(SetWorldTransform(hdc, &xf))) return NULL;
+    return Py_BuildValue("O", Py_None);
+}
+static PyObject* PyTransparentBlt(PyObject* _, PyObject* args) {
+    HDC hdcDest;
+    INT xDest;
+    INT yDest;
+    INT wDest;
+    INT hDest;
+    HDC hdcSrc;
+    INT xSrc;
+    INT ySrc;
+    INT wSrc;
+    INT hSrc;
+    UINT transparentColor;
+
+    if (!PyArg_ParseTuple(args, "liiiiliiiiI", &hdcDest, &xDest, &yDest, &wDest, &hDest, &hdcSrc, &xSrc, &ySrc, &wSrc, &hSrc, &transparentColor)) return NULL;
+    
+    if (RaiseExceptionCheckInvalid(TransparentBlt(hdcDest, xDest, yDest, wDest, hDest, hdcSrc, xSrc, ySrc, wSrc, hSrc, transparentColor))) return NULL;
+    return Py_BuildValue("O", Py_None);
+}
+static PyObject* PySetViewportExtEx(PyObject* _, PyObject* args) {
+    HDC hdc;
+    INT x;
+    INT y;
+    SIZE szOut;
+    if (!PyArg_ParseTuple(args, "lii", &hdc, &x, &y)) return NULL;
+    if (RaiseExceptionCheckInvalid(SetViewportExtEx(hdc, x, y, &szOut))) return NULL;
+    return Py_BuildValue("(ll)", szOut.cx, szOut.cy);
+}
+static PyObject* PySetViewportOrgEx(PyObject* _, PyObject* args) {
+    HDC hdc;
+    INT x;
+    INT y;
+    POINT szOut;
+    if (!PyArg_ParseTuple(args, "lii", &hdc, &x, &y)) return NULL;
+    if (RaiseExceptionCheckInvalid(SetViewportOrgEx(hdc, x, y, &szOut))) return NULL;
+    return Py_BuildValue("(ll)", szOut.x, szOut.y);
+}
+static PyObject* PySetWindowExtEx(PyObject* _, PyObject* args) {
+    HDC hdc;
+    INT x;
+    INT y;
+    SIZE szOut;
+    if (!PyArg_ParseTuple(args, "lii", &hdc, &x, &y)) return NULL;
+    if (RaiseExceptionCheckInvalid(SetWindowExtEx(hdc, x, y, &szOut))) return NULL;
+    return Py_BuildValue("(ll)", szOut.cx, szOut.cy);
+}
+static PyObject* PySetWindowOrgEx(PyObject* _, PyObject* args) {
+    HDC hdc;
+    INT x;
+    INT y;
+    POINT szOut;
+    if (!PyArg_ParseTuple(args, "lii", &hdc, &x, &y)) return NULL;
+    if (RaiseExceptionCheckInvalid(SetWindowOrgEx(hdc, x, y, &szOut))) return NULL;
+    return Py_BuildValue("(ll)", szOut.x, szOut.y);
+}
+static PyObject* PySetGraphicsMode(PyObject* _, PyObject* args) {
+    HDC hdc;
+    INT iMode;
+    if (!PyArg_ParseTuple(args, "li", &hdc, &iMode)) return NULL;
+    INT res = SetGraphicsMode(hdc, iMode);
+    if (RaiseExceptionCheckInvalid(res)) return NULL;
+    return Py_BuildValue("i", res);
+}
+static PyObject* PySetColorAdjustment(PyObject* _, PyObject* args) {
+    HDC hdc;
+    COLORADJUSTMENT c;
+    if (!PyArg_ParseTuple(args, "l(HHHHHHHhhhh)",
+        &hdc,
+        &c.caFlags, &c.caIlluminantIndex, &c.caRedGamma, &c.caGreenGamma, &c.caBlueGamma, &c.caReferenceBlack,
+        &c.caReferenceWhite, &c.caContrast, &c.caBrightness, &c.caColorfulness, &c.caRedGreenTint
+    )) return NULL;
+    c.caSize = sizeof(COLORADJUSTMENT);
+    if (RaiseExceptionCheckInvalid(SetColorAdjustment(hdc, &c))) return NULL;
+    return Py_BuildValue("O", Py_None);
+}
 static PyMethodDef module_methods[] = {
+    { "SetColorAdjustment", PySetColorAdjustment, METH_VARARGS },
+    { "SetGraphicsMode", PySetGraphicsMode, METH_VARARGS },
+    { "SetWindowOrgEx", PySetWindowOrgEx, METH_VARARGS },
+    { "SetWindowExtEx", PySetWindowExtEx, METH_VARARGS },
+    { "SetViewportExtEx", PySetViewportExtEx, METH_VARARGS },
+    { "SetViewportOrgEx", PySetViewportOrgEx, METH_VARARGS },
+    { "TransparentBlt", PyTransparentBlt, METH_VARARGS },
+    { "SetWorldTransform", PySetWorldTransform, METH_VARARGS },
+    { "ModifyWorldTransform", PyModifyWorldTransform, METH_VARARGS },
+    { "OffsetViewportOrgEx", PyOffsetViewportOrgEx, METH_VARARGS },
+    { "OffsetWindowOrgEx", PyOffsetWindowOrgEx, METH_VARARGS },
+    { "OffsetClipRgn", PyOffsetClipRgn, METH_VARARGS },
+    { "OffsetRgn", PyOffsetRgn, METH_VARARGS },
+    { "PolyPolyline", PyPolyPolyline, METH_VARARGS },
+    { "PolylineTo", PyPolylineTo, METH_VARARGS },
+    { "Polyline", PyPolyline, METH_VARARGS },
+    { "PolyDraw", PyPolyDraw, METH_VARARGS },
+    { "PolyPolygon", PyPolyPolygon, METH_VARARGS },
+    { "Polygon", PyPolygon, METH_VARARGS },
+    { "MoveToEx", PyMoveToEx, METH_VARARGS },
+    { "PolyBezierTo", PyPolyBezierTo, METH_VARARGS },
+    { "PolyBezier", PyPolyBezier, METH_VARARGS },
+    { "CreatePolyPolygonRgn", PyCreatePolyPolygonRgn, METH_VARARGS },
+    { "PolyTextOut",  PyPolyTextOutW, METH_VARARGS },
+    { "PolyTextOutW", PyPolyTextOutW, METH_VARARGS },
+    { "PolyTextOutA", PyPolyTextOutA, METH_VARARGS },
+    { "RectVisible", PyRectVisible, METH_VARARGS },
+    { "RectInRegion", PyRectInRegion, METH_VARARGS },
+    { "GetTextExtentPoint32",  PyGetTextExtentPoint32W, METH_VARARGS },
+    { "GetTextExtentPoint32W", PyGetTextExtentPoint32W, METH_VARARGS },
+    { "GetTextExtentPoint32A", PyGetTextExtentPoint32A, METH_VARARGS },
+    { "CreateFont",  PyCreateFontW, METH_VARARGS },
+    { "CreateFontW", PyCreateFontW, METH_VARARGS },
+    { "CreateFontA", PyCreateFontA, METH_VARARGS },
+    { "PtInRegion", PyPtInRegion, METH_VARARGS },
+    { "PtVisible", PyPtVisible, METH_VARARGS },
     {"CreateDIBSection", (PyCFunction)PyCreateDIBSection, METH_VARARGS},
     {"PatBlt", (PyCFunction)PyPatBlt, METH_VARARGS},
     {"DeleteDC", (PyCFunction)PyDeleteDC, METH_VARARGS},
